from distutils.core import setup
setup(
  name = 'Pylinear',
  packages = ['Pylinear'], # this must be the same as the name above
  version = '0.1',
  description = 'A lightweight python package for solving linear algebra problems',
  author = 'Max Poon',
  author_email = 'maxpanziyuan@gmail.com',
  url = 'https://github.com/MaxPoon/Pylinear', # use the URL to the github repo
  download_url = 'https://github.com/MaxPoon/Pylinear/tarball/0.1', # I'll explain this in a second
  keywords = ['Pylinear', 'linear', 'algebra'], # arbitrary keywords
  classifiers = [],
)